"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* --------------------------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
const vscode_jsonrpc_1 = require("vscode-jsonrpc");
const main_1 = require("./main");
const protocol_proposed_1 = require("./protocol.proposed");
const Is = require("./utils/is");
const vscode_uri_1 = require("vscode-uri");
exports.WorkspaceFoldersFeature = (Base) => {
    return class extends Base {
        initialize(capabilities) {
            let workspaceCapabilities = capabilities.workspace;
            if (workspaceCapabilities.workspaceFolders) {
                this._onDidChangeWorkspaceFolders = new vscode_jsonrpc_1.Emitter();
                this.connection.onNotification(protocol_proposed_1.DidChangeWorkspaceFolders.type, (params) => {
                    this._onDidChangeWorkspaceFolders.fire(params.event);
                });
            }
        }
        getWorkspaceFolders() {
            return this.connection.sendRequest(protocol_proposed_1.GetWorkspaceFolders.type);
        }
        getWorkspaceFolder(uri) {
            return this.connection.sendRequest(protocol_proposed_1.GetWorkspaceFolder.type, uri);
        }
        get onDidChangeWorkspaceFolders() {
            if (!this._onDidChangeWorkspaceFolders) {
                throw new Error('Client doesn\'t support sending workspace folder change events.');
            }
            if (!this._unregistration) {
                this._unregistration = this.connection.client.register(protocol_proposed_1.DidChangeWorkspaceFolders.type);
            }
            return this._onDidChangeWorkspaceFolders.event;
        }
    };
};
exports.GetConfigurationFeature = (Base) => {
    return class extends Base {
        getGlobalConfiguration(sections) {
            return this._getConfiguration(undefined, sections);
        }
        getScopedConfiguration(scopeUris, sections) {
            if (Is.string(scopeUris)) {
                return this._getConfiguration(scopeUris, sections);
            }
            else if (scopeUris instanceof vscode_uri_1.default) {
                return this._getConfiguration(scopeUris.toString(), sections);
            }
            else if (Array.isArray(scopeUris)) {
                if (scopeUris.length === 0) {
                    return this._getConfiguration([], sections);
                }
                else {
                    return this._getConfiguration(scopeUris.map(element => Is.string(element) ? element : element.toString()), sections);
                }
            }
            else {
                return this._getConfiguration(undefined, undefined);
            }
        }
        _getConfiguration(scopeUris, sections) {
            let params = {};
            if (Is.string(scopeUris)) {
                params.scopeUris = [scopeUris];
            }
            else if (Is.stringArray(scopeUris)) {
                params.scopeUris = scopeUris;
            }
            if (Is.string(sections)) {
                params.sections = [sections];
            }
            else if (Is.stringArray(sections)) {
                params.sections = sections;
            }
            return this.connection.sendRequest(protocol_proposed_1.GetConfigurationRequest.type, params).then((result) => {
                let uriIsString = Is.string(scopeUris);
                let uriIsArray = Is.stringArray(scopeUris);
                let sectionIsString = Is.string(sections);
                let sectionIsArray = Is.stringArray(sections);
                if ((scopeUris === void 0 || scopeUris === null || uriIsString) && (sections === void 0 || sections === null || sectionIsString)) {
                    return result[0][0];
                }
                else if (uriIsArray && sectionIsString) {
                    return result.map(elements => elements[0]);
                }
                else if (uriIsString && sectionIsArray) {
                    return result[0];
                }
                else if (uriIsArray && sectionIsArray) {
                    return result;
                }
                else {
                    return Promise.reject(new Error('Invalid arguments provided for get configuration request'));
                }
            });
        }
    };
};
exports.ProposedProtocol = {
    __brand: 'features',
    workspace: main_1.combineWorkspaceFeatures(exports.WorkspaceFoldersFeature, exports.GetConfigurationFeature)
};
